--[[
Name: LibAuraInfo-1.0
Author(s): Cyprias (cyprias@gmail.com)
Documentation: http://www.wowace.com/addons/libaurainfo-1-0/
SVN: svn://svn.wowace.com/wow/libaurainfo-1-0/mainline/trunk
Description: Database of spellID's duration and debuff type.
Dependencies: LibStub
]]

local _G = _G
local print = print
local UnitAura = UnitAura
local _
local math_floor = math.floor
local strsplit = strsplit
local tonumber = tonumber
local UnitIsUnit = UnitIsUnit
local UnitExists = UnitExists

local MAJOR, MINOR = "LibAuraInfo-1.0", 1
if not LibStub then error(MAJOR .. " requires LibStub.") return end

local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end


local DEBUG = false
--[===[@debug@
DEBUG = true
--@end-debug@]===]

local function debugPrint(...)
	if DEBUG then
		print(...)
	end
end

--Save debuffType as a number, then return as a string when requested.
local debuffTypes = {
	Magic = 1,
	Disease = 2,
	Poison = 3,
	Curse = 4,
	[1] = "Magic",
	[2] = "Disease",
	[3] = "Poison",
	[4] = "Curse",
}

--Return the duration of a spell.
function lib:GetDuration(spellID)
	if lib.auraInfo[spellID] then
		local duration, debuffType = strsplit(";", lib.auraInfo[spellID])
		return tonumber(duration or 0)
	end
	return nil
end

--Return the debuff type of a spell.
function lib:GetDebuffType(spellID)
	if lib.auraInfo[spellID] then
		local duration, debuffType = strsplit(";", lib.auraInfo[spellID])
		if debuffType then
			if debuffType == 0 then
				return "None"
			else
				return debuffTypes[debuffType] or "unknown"
			end
		end
		return "None"
	end
	return nil
end

local LAI_DB
local function Round(num, zeros)
	return math_floor( num * 10 ^ (zeros or 0) + 0.5 ) / 10 ^ (zeros or 0)
end
local function CheckUnitAuras(unitID, filterType)
--~ 	name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId  = UnitAura("unit", index or "name"[, "rank"[, "filter"]]) 
	local i = 1;
	local name, rank, dispelType, duration, isStealable, spellID
	while true do 
		name, rank, _, _, dispelType, duration, _, _, isStealable, _, spellID = UnitAura(unitID, i, filterType)
		if not name then break end
		
		--[===[@debug@
		if not duration  then
			debugPrint("duration", spellID, name, rank, duration)
		end
		
		if isStealable then
			debugPrint("isStealable", spellID, name, rank, isStealable)
		end
		--@end-debug@]===]

		if not lib.auraInfo[spellID] then
			duration = Round(duration)
			lib.auraInfo[spellID] = (duration or 0) .. ";" .. (dispelType and debuffTypes[dispelType] or 0)--add it temporarily.
			
			--[===[@debug@
			if not LAI_DB[spellID] then
				--Save the spell info to a saved variable to add it later. End users won't execute this code.
				LAI_DB[spellID] = lib.auraInfo[spellID]    .."  --"..name .. (rank and rank ~= "" and " ("..rank..")" or "") .."*"
				debugPrint("Missing info on", spellID, dispelType, duration, name, rank)
			end
			--@end-debug@]===]
		end
		i = i + 1
	end
end
local function OnEvent(self, event, ...)
	--I plan on removing this once we get near to release.
	if event == "UPDATE_MOUSEOVER_UNIT" then
		CheckUnitAuras("mouseover", "HELPFUL")
		CheckUnitAuras("mouseover", "HARMFUL")
	elseif event == "PLAYER_TARGET_CHANGED" then
		CheckUnitAuras("target", "HELPFUL")
		CheckUnitAuras("target", "HARMFUL")
	elseif event == "UNIT_TARGET" then
		local unitID = ...
		if not UnitIsUnit(unitID, "player") and UnitExists(unitID.."target") then
			CheckUnitAuras(unitID.."target", "HELPFUL")
			CheckUnitAuras(unitID.."target", "HARMFUL")
		end
	elseif event == "UNIT_AURA" then
		local unitID = ...
		if UnitExists(unitID) then
			CheckUnitAuras(unitID, "HELPFUL")
			CheckUnitAuras(unitID, "HARMFUL")
		end
	elseif event == "PLAYER_LOGOUT" then
		--[===[@debug@
		_G.LAI_DB = LAI_DB
		--@end-debug@]===]
	elseif event == "ADDON_LOADED" or event == "PLAYER_LOGIN" then
		--[===[@debug@
		LAI_DB = _G.LAI_DB or {}
		self:RegisterEvent('PLAYER_LOGOUT')
		--@end-debug@]===]
		self:RegisterEvent('UPDATE_MOUSEOVER_UNIT')
		self:RegisterEvent('PLAYER_TARGET_CHANGED')
		self:RegisterEvent('UNIT_TARGET')
		self:RegisterEvent('UNIT_AURA')
		
	end
end
lib.frame = lib.frame or CreateFrame("Frame")
lib.frame:SetScript("OnEvent", OnEvent)
lib.frame:RegisterEvent('ADDON_LOADED')
lib.frame:RegisterEvent('PLAYER_LOGIN')